" Begin vundle

set nocompatible
syntax on
filetype off

if has("nvim")
	tnoremap <Esc> <C-\><C-n>
	tnoremap <Esc><Esc> <C-\><C-n>:close<cr>
endif

let mapleader = ","
let maplocalleader = "\\"

" set the runtime path to include Vundle and initialize
set rtp+=~/.vim/bundle/Vundle.vim
call vundle#begin()
Plugin 'VundleVim/Vundle.vim'			" Does this require explanation?

Plugin 'bling/vim-airline'				" Status bar
Plugin 'vim-airline/vim-airline-themes'	" Status bar themes
Plugin 'edkolev/tmuxline.vim'			" tmux status bar
Plugin 'scrooloose/nerdtree'			" File browser sidebar
Plugin 'scrooloose/nerdcommenter'		" Extra comment shortcuts
Plugin 'jistr/vim-nerdtree-tabs'		" Sensible nerdtree behavior
Plugin 'mattn/emmet-vim'				" HTML/CSS expansions
Plugin 'kien/ctrlp.vim'					" Open surrounding files
Plugin 'scrooloose/syntastic'			" Syntax chacker
Plugin 'airblade/vim-gitgutter'			" git info in statusbar
Plugin 'majutsushi/tagbar'				" File tag sidebar

Plugin 'hynek/vim-python-pep8-indent'	" Python indentation fixes
Plugin 'jelera/vim-javascript-syntax'	" JS syntax fixes
Plugin 'sirtaj/vim-openscad'			" openscad syntax file

call vundle#end()
filetype plugin indent on

" the One True Indentation Setting
" if you don't like it, get out
set ts=4 sts=4 sw=4 expandtab

""""""""""""""""""""
"  plugin options  "
"\________________/"

" airline
let g:airline#extensions#tabline#enabled = 1
let g:airline_theme = 'term'
if has("nvim") || &term =~? "256color" || has("gui_running")
	let g:airline_powerline_fonts = 1
	set t_ut=
	if !exists('g:airline_symbols')
		let g:airline_symbols = {}
	endif
	let g:airline_symbols.maxlinenr = ''
endif

" NERDTree
autocmd bufenter * if (winnr("$") == 1 && exists("b:NERDTreeType") && b:NERDTreeType == "primary") | q | endif
"let g:nerdtree_tabs_open_on_console_startup=1
map <C-p> :NERDTreeToggle<CR>
nmap <leader>p :NERDTreeFind<CR>

" Emmet (use <C-y>, to activate)
"let g:user_emmet_leader_key='<C->'
let g:user_emmet_install_global = 0
autocmd FileType html,css EmmetInstall

" CtrlP
let g:ctrlp_map = '<leader>,'
let g:ctrlp_cmd = 'CtrlP'
cal ctrlp#clra()
let g:ctrlp_switch_buffer = 1
nmap <leader>. :CtrlPClearCache<CR>:CtrlP<CR>
nmap <leader>m :CtrlPBuffer<CR>

" nerdcomment
" invert comment on the current line/selection
nmap <leader>/ :call NERDComment(0, "invert")<cr>
vmap <leader>/ :call NERDComment(0, "invert")<cr>

" syntastic
set statusline+=%#warningmsg#
set statusline+=%{SyntasticStatuslineFlag()}
set statusline+=%*

let g:syntastic_always_populate_loc_list = 1
let g:syntastic_auto_loc_list = 1
let g:syntastic_check_on_open = 0
let g:syntastic_check_on_wq = 0

nnoremap <silent> <C-d> :lclose<CR>:bdelete<CR>
cabbrev <silent> bd <C-r>=(getcmdtype()==#':' && getcmdpos()==1 ? 'lclose\|bdelete' : 'bd')<CR>

let g:syntastic_mode_map = {'mode': 'passive', 'active_filetypes': [], 'passive_filetypes': ['html'] }
nmap gr :SyntasticCheck<CR>
nmap gc :SyntasticReset<CR>

" tagbar
nmap gb :TagbarToggle<CR>
nmap <F9> :TagbarToggle<CR>
let g:tagbar_autoclose = 1

" End vundle

" mouse support
if has('mouse')
	set mouse=a
endif

" swapfiles without clutter +1
if has("win32") || has("win64")
   set directory=$TMP
else
   set directory=/tmp
end 

" line numbers
set nu
set relativenumber
" permanent statusline
set laststatus=2
" hidden buffers
set hidden
" color scheme
colorscheme Tomorrow-Night-Eighties
"set t_Co=256

if has("gui_running")
	colorscheme Tomorrow-Night-Eighties-bg
	set guifont=DejaVu\ Sans\ Mono\ for\ Powerline\ 10
	let g:airline_theme = 'tomorrow'
endif

" history
set history=250
" actually, history sucks, don't persist it
if has("nvim")
	set viminfo+=n~/.tmp/.nviminfo
else
	set viminfo+=n~/.tmp/.viminfo
endif

" CUSTOM BINDINGS

" how could you not
nnoremap ; :
vnoremap ; :
" look up word in Zeal (fixed by copying auscompgeek's)
nnoremap gu :!zeal -style=gtk --query "<cword>" &>/dev/null & <cr><cr>
if has("nvim")
	nnoremap gt :rightbelow vertical split <bar> :term dasht "<cword>" <cr>
else
	nnoremap gt ::exe '!dasht ' . expand('<cword>') . ' ' . &ft <cr>
endif
" comment line
nmap gy <leader>c<Space>
vmap gy <leader>c<Space>

" buffer management
map <Leader>a :bprev<Return>
map <Leader>s :bnext<Return>
map <Leader>d :bd<Return>
map <Leader>f :b 

" save as sudo
cmap w!! w !sudo tee > /dev/null %

" open in browser
nmap gs :!xdg-open %:p &<CR><CR>

" the typical vimbits.com stuff

" keep selection after in/outdent
vnoremap < <gv
vnoremap > >gv
" better navigation of wrapped lines
nnoremap j gj
nnoremap k gk
" increment/decrement
nnoremap + <C-a>
nnoremap - <C-x>
" easy split navigation
nnoremap <C-h> <C-w>h
nnoremap <C-j> <C-w>j
nnoremap <C-k> <C-w>k
nnoremap <C-l> <C-w>l

" bulky functions

" XML linting
function! DoFormatXML() range
	" Save the file type
	let l:origft = &ft

	" Clean the file type
	set ft=

	" Add fake initial tag (so we can process multiple top-level elements)
	exe ":let l:beforeFirstLine=" . a:firstline . "-1"
	if l:beforeFirstLine < 0
		let l:beforeFirstLine=0
	endif
	exe a:lastline . "put ='</PrettyXML>'"
	exe l:beforeFirstLine . "put ='<PrettyXML>'"
	exe ":let l:newLastLine=" . a:lastline . "+2"
	if l:newLastLine > line('$')
		let l:newLastLine=line('$')
	endif

	" Remove XML header
	exe ":" . a:firstline . "," . a:lastline . "s/<\?xml\\_.*\?>\\_s*//e"

	" Recalculate last line of the edited code
	let l:newLastLine=search('</PrettyXML>')

	" Execute external formatter
	exe ":silent " . a:firstline . "," . l:newLastLine . "!xmllint --noblanks --format --recover -"

	" Recalculate first and last lines of the edited code
	let l:newFirstLine=search('<PrettyXML>')
	let l:newLastLine=search('</PrettyXML>')

	" Get inner range
	let l:innerFirstLine=l:newFirstLine+1
	let l:innerLastLine=l:newLastLine-1

	" Remove extra unnecessary indentation
	exe ":silent " . l:innerFirstLine . "," . l:innerLastLine "s/^  //e"

	" Remove fake tag
	exe l:newLastLine . "d"
	exe l:newFirstLine . "d"

	" Put the cursor at the first line of the edited code
	exe ":" . l:newFirstLine

	" Restore the file type
	exe "set ft=" . l:origft
endfunction
command! -range=% FormatXML <line1>,<line2>call DoFormatXML()

nmap <silent> <leader>x :%FormatXML<CR>
vmap <silent> <leader>x :FormatXML<CR>
